/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.dao.impl;

import dk.stamkort.dao.PropertiesDAO;
import dk.stamkort.dao.exceptions.DAOException;
import dk.stamkort.dao.impl.abstr.AbstractDAOImpl;
import dk.stamkort.domain.PropertiesVO;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.stereotype.Repository;

@Repository
public class PropertiesDAOImpl extends AbstractDAOImpl implements PropertiesDAO {

    public static final String TABLE_NAME = "Properties";

    @Override
    public PropertiesVO getProperty(String key) throws DAOException {
        return fetchSingleRowMatchingSingleColumn("propertyKey", key);
    }

    @Override
    public void updateProperty(String key, String value) throws DAOException {
        try {
            String sql = null;
            if (getProperty(key) != null) {
                sql = "UPDATE " + TABLE_NAME + " SET propertyValue = :propertyValue WHERE propertyKey = :propertyKey";
            } else {
                sql = "INSERT INTO " + TABLE_NAME + " (propertyKey, propertyValue) values (:propertyKey, :propertyValue)";
            }
            final MapSqlParameterSource parameters = new MapSqlParameterSource();
            parameters.addValue("propertyKey", key);
            parameters.addValue("propertyValue", value);
            executeUpdate(sql, parameters);
        } catch (DAOException e) {
            throw new DAOException("An error occurred updating person with propertyKey: " + key + " and propertyValue: " + value, e);
        }
    }

    @Override
    public String getTableName() {
        return TABLE_NAME;
    }

    @Override
    protected String getIdColumnName() {
        return "PropertiesPID";
    }

    @Override
    protected RowMapper getDefaultRowMapper() {
        return new PropertiesRowMapper();
    }

}
