/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.stamkort.util.SlaLogger;
import oasis.names.tc.ebxml_regrep.xsd.rs._3.RegistryError;
import oasis.names.tc.ebxml_regrep.xsd.rs._3.RegistryErrorList;
import oasis.names.tc.ebxml_regrep.xsd.rs._3.RegistryResponseType;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import java.io.IOException;
import java.io.StringWriter;

public class DDSClient extends DGWSClient {

    private static final Logger log = LogManager.getLogger(DDSClient.class);

    @Autowired
    private dk.sdsd.nsp.slalog.api.SLALogger slaLogFactory;

    private SlaLogger slaLogger;

    public DDSClient(String name, String keystorePath, String keystorePassword, String orgId, String orgName, String stsEndpoint, String endpoint, int connectTimeoutMillis, int readTimeoutMillis, String itSystemName, boolean isProduction) {
        super(keystorePath, keystorePassword, orgId, orgName, stsEndpoint, endpoint, connectTimeoutMillis, readTimeoutMillis, itSystemName, isProduction);
        this.slaLogger = new SlaLogger(name);
    }

    public void setEndpoint(String endpoint) {
        super.endpoint = endpoint;
    }

    protected RegistryResponseType toRegistryResponse(String responseXml) throws JAXBException, IOException, SOAPException {
        SOAPBody body = getSoapBody(responseXml);
        if (body.getFault() != null) {
            log.warn("Error calling DDS: " + body.getFault().getFaultString());
            return null;
        }

        JAXBElement<RegistryResponseType> jaxbResponse = getUnmarshaller(RegistryResponseType.class).unmarshal(body.getFirstChild(), RegistryResponseType.class);
        return jaxbResponse.getValue();
    }

    protected String getErrorsFromDDSResponse(RegistryResponseType registryResponse) {
        if (registryResponse == null) {
            return null;
        }
        RegistryErrorList errorList = registryResponse.getRegistryErrorList();
        if (errorList == null || errorList.getRegistryError() == null || errorList.getRegistryError().size() == 0) {
            return null;
        } else {
            StringWriter sw = new StringWriter();
            for (RegistryError error : errorList.getRegistryError()) {
                sw.append("[" + error.getCodeContext() + "]");
            }
            return sw.toString();
        }
    }

    protected String sendRequest(String requestXml, String soapAction) throws IOException {
        SLALogItem slaLogItem = slaLogFactory.createLogItem("fsk.dds", "fsk.dds");
        long start = System.currentTimeMillis();
        long startNano = System.nanoTime();
        try {
            String response = transport.sendRequest(endpoint, soapAction, requestXml);
            slaLogger.addSlaLogWithoutRequestContext(slaLogItem, startNano, start, null);
            return response;
        } catch (RuntimeException e) {
            slaLogger.addSlaLogWithoutRequestContext(slaLogItem, startNano, start, e.getMessage());
            throw e;
        }
    }

}
