/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations;

import dk.stamkort.domain.YderVO;
import dk.sundhedsdatastyrelsen.fsk._2019._08._01.GetPersonalDataCardResponse;
import oio.medcom.cprservice._1_0.GetPersonWithHealthCareInformationOut;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class RegisterData {
    private String cprnr;
    private GetPersonWithHealthCareInformationOut scesData;
    private List<ChildCustodyCustodyData> childCustodyCustodyData = new ArrayList<>();
    private List<CustodyByCustodyData> custodyByCustodyData = new ArrayList<>();
    private GetPersonalDataCardResponse personalDataCard;
    private Boolean hasLivingWill;
    private Boolean hasTreatmentWill;
    private Boolean hasOrganDonorRegistration;
    private Map<String, YderVO> yderInfo;

    public RegisterData(String cprnr) {
        this.cprnr = cprnr;
        yderInfo = new HashMap<>();
    }

    public String getCprnr() {
        return cprnr;
    }

    public void setCprnr(String cprnr) {
        this.cprnr = cprnr;
    }

    public GetPersonWithHealthCareInformationOut getScesData() {
        return scesData;
    }

    public void setScesData(GetPersonWithHealthCareInformationOut scesData) {
        this.scesData = scesData;
    }

    public List<ChildCustodyCustodyData> getChildCustodyCustodyData() {
        return childCustodyCustodyData;
    }

    public void addChildCustodyCustodyData(ChildCustodyCustodyData childCustodyCustodyData) {
        this.childCustodyCustodyData.add(childCustodyCustodyData);
    }

    public List<CustodyByCustodyData> getCustodyByCustodyData() {
        return custodyByCustodyData;
    }

    public void addCustodyByCustodyData(CustodyByCustodyData custodyByCustodyData) {
        this.custodyByCustodyData.add(custodyByCustodyData);
    }

    public GetPersonalDataCardResponse getPersonalDataCard() {
        return personalDataCard;
    }

    public void setPersonalDataCard(GetPersonalDataCardResponse personalDataCard) {
        this.personalDataCard = personalDataCard;
    }

    public Boolean getHasLivingWill() {
        return hasLivingWill;
    }

    public void setHasLivingWill(Boolean hasLivingWill) {
        this.hasLivingWill = hasLivingWill;
    }

    public Boolean getHasTreatmentWill() {
        return hasTreatmentWill;
    }

    public void setHasTreatmentWill(Boolean hasTreatmentWill) {
        this.hasTreatmentWill = hasTreatmentWill;
    }

    public Boolean getHasOrganDonorRegistration() {
        return hasOrganDonorRegistration;
    }

    public void setHasOrganDonorRegistration(Boolean hasOrganDonorRegistration) {
        this.hasOrganDonorRegistration = hasOrganDonorRegistration;
    }

    public YderVO getYder(String ydernr) {
        return yderInfo.get(ydernr);
    }

    public void addYder(String yderNr, YderVO yder) {
        if (yderNr != null && yder != null) {
            yderInfo.put(yderNr, yder);
        }
    }
}
