/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations.dds;

import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;
import dk.stamkort.dao.exceptions.DDSPermanentException;
import dk.stamkort.dao.exceptions.DDSTemporaryException;
import dk.stamkort.integrations.DDSClient;
import dk.stamkort.util.Properties;
import oasis.names.tc.ebxml_regrep.xsd.lcm._3.SubmitObjectsRequest;
import oasis.names.tc.ebxml_regrep.xsd.rs._3.RegistryResponseType;
import org.springframework.beans.factory.annotation.Autowired;
import org.w3c.dom.Document;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPException;
import java.io.IOException;
import java.net.MalformedURLException;

import static dk.stamkort.integrations.dds.XdsDocumentEntryConstants.XDS_STATUS_SUCCESS;

public class DDSRegistryUpdateClientImpl extends DDSClient implements DDSRegistryUpdateClient {

    @Autowired
    Properties properties;

    public DDSRegistryUpdateClientImpl(String keystorePath, String keystorePassword, String orgId, String orgName, String stsEndpoint, String endpoint, int connectTimeoutMillis, int readTimeoutMillis, String itSystemName, boolean isProduction) throws MalformedURLException {
        super(DDSRegistryUpdateClientImpl.class.getName(), keystorePath, keystorePassword, orgId, orgName, stsEndpoint, endpoint, connectTimeoutMillis, readTimeoutMillis, itSystemName, isProduction);
    }

    @Override
    public void deprecateDocumentUsingDdsId(String patientCPR, String ddsId) throws DDSPermanentException, DDSTemporaryException {
        try {
            XDSRequestHelper requestHelper = new XDSRequestHelper();
            String patientId = requestHelper.getPatientId(patientCPR);
            SubmitObjectsRequest submitObjectsRequest = requestHelper.create();

            requestHelper.addSubmissionSetForUpdate(submitObjectsRequest, properties.getDdsHomeCommunityId(), patientId, ddsId, properties.getDdsHomeCommunityId(), properties.getDdsTypeCode());
            requestHelper.addAssociationForUpdate(submitObjectsRequest, ddsId);

            Request request = prepareRequest();
            request.setBody(toXml(submitObjectsRequest, new QName("urn:oasis:names:tc:ebxml-regrep:xsd:lcm:3.0", "SubmitObjectsRequest")));
            Document soapRequestDOM = request.serialize2DOMDocument();
            String requestXml = XmlUtil.node2String(soapRequestDOM);
            String responseXml = sendRequest(requestXml, "urn:ihe:iti:2010:UpdateDocumentSet");

            RegistryResponseType registryResponseType = toRegistryResponse(responseXml);
            if (registryResponseType == null || !XDS_STATUS_SUCCESS.equals(registryResponseType.getStatus())) {
                throw new DDSPermanentException("Unexpected DDS response for DDS id " + ddsId + ": " + getErrorsFromDDSResponse(registryResponseType));
            }
        } catch (JAXBException | SOAPException | IOException e) {
            throw new DDSTemporaryException("Error deprecating DDS document with id " + ddsId, e);
        }
    }

}
