/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations.dds;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class JobStatus {

    private static final Logger log = LogManager.getLogger(SyncJob.class);

    private int numberOfErrors = 0;

    private int allowedNumberOfErrors;

    public JobStatus(int allowedNumberOfErrors) {
        this.allowedNumberOfErrors = allowedNumberOfErrors;
    }

    void increaseNumberOfErrors() {
        numberOfErrors++;
    }

    public boolean tooManyErrors() {
        return numberOfErrors > allowedNumberOfErrors;
    }

    public String errorText() {
        return "Max number of errors (" + numberOfErrors + ") exceeded for SyncJob (max allowed is " + allowedNumberOfErrors + ")";
    }

    public String warnText() {
        return "Currently " + numberOfErrors + " error encountered for SyncJob (max allowed is " + allowedNumberOfErrors + ")";
    }
}
