/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.util;

import dk.sds.dgws_idws_proxy.security.ProxyData;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import org.w3c.dom.Element;

import java.util.HashMap;
import java.util.Map;

public class RequestContextVO {
    private static final ThreadLocal<RequestContextVO> currentRequestContext = new ThreadLocal<RequestContextVO>();

    private SLALogItem slaLogItem;
    private ProxyData proxyData;
    private Element securityHeader;
    private Element medcomHeader;
    private Map<String,String> auditLogParameters;

    public static RequestContextVO get() {
        RequestContextVO requestContext = currentRequestContext.get();
        if (requestContext == null) {
            requestContext = new RequestContextVO();
            currentRequestContext.set(requestContext);
        }
        return requestContext;
    }

    public SLALogItem getSlaLogItem() {
        return slaLogItem;
    }

    public void setSlaLogItem(SLALogItem slaLogItem) {
        this.slaLogItem = slaLogItem;
    }

    public ProxyData getProxyData() {
        return proxyData;
    }

    public void setProxyData(ProxyData proxyData) {
        this.proxyData = proxyData;
    }

    public Element getSecurityHeader() {
        return securityHeader;
    }

    public void setSecurityHeader(Element securityHeader) {
        this.securityHeader = securityHeader;
    }

    public Element getMedcomHeader() {
        return medcomHeader;
    }

    public void setMedcomHeader(Element medcomHeader) {
        this.medcomHeader = medcomHeader;
    }

    public void addAuditLogParameter(String key, String value) {
        if (key == null) {
            return;
        }
        if (auditLogParameters == null) {
            auditLogParameters = new HashMap<>();
        }
        auditLogParameters.put(key, value);
    }

    public Map<String,String> getAuditLogParameters() {
        return auditLogParameters;
    }

}
