/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.syncjob;

import dk.stamkort.integrations.dds.SyncJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/syncjob")
public class SyncJobController {

    @Autowired
    private SyncJob syncJob;

    @GetMapping(value = "/start", produces = "application/json")
    public Map<String, String> SyncJobStart() {

        DateTimeFormatter FOMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime syncJobStartDateTime = LocalDateTime.now();

        syncJob.runSyncPersonIds();

        HashMap<String, String> map = new HashMap<>();
        map.put("timeOfLastSyncJobStart", FOMATTER.format(syncJobStartDateTime));
        return map;
    }

    @GetMapping(value = "/status", produces = "application/json")
    public Map<String, String> SyncJobStatus() {

        HashMap<String, String> map = new HashMap<>();
        map.put("timeOfLastExecution", syncJob.timeOfLastExecution() == null ? "" : syncJob.timeOfLastExecution().toString());
        map.put("lastExecutionSucceeded", syncJob.lastExecutionSucceeded() == null ? "" : syncJob.lastExecutionSucceeded().toString());
        map.put("lastException", syncJob.lastException() == null ? "" : syncJob.lastException().getMessage());
        return map;
    }
}
