/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort;

import dgws.SosiUtil;
import dk.sosi.seal.model.AuthenticationLevel;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;


@RunWith(SpringRunner.class)
@SpringBootTest
public class PersonalDataCardApplicationTests {
    private static final Logger log = LogManager.getLogger(PersonalDataCardApplicationTests.class);
    private static final String ENCODING = "UTF-8";

    @Test
    public void contextLoads() {
    }

    @Test
    @Ignore // Utility for making a direct call to the RetrieveDocumentSet web service. Not to be run as part of usual unit tests
    public void callOnDemandService() {
        try {
            InputStream stream = getClass().getResourceAsStream("/testrequest.xml");
            SOAPMessage soapMessage = MessageFactory.newInstance().createMessage(null, stream);

            log.debug("Initializing SosiUtil");
            try {
                SosiUtil sosiUtil = new SosiUtil();
                log.debug("SosiUtil initialized");
                sosiUtil.addSoapHeader(soapMessage, AuthenticationLevel.MOCES_TRUSTED_USER);
            } catch (Throwable ex) {
                log.error("SosiUtil initialization failed", ex);
            }

            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            soapMessage.writeTo(baos);
            String xml = baos.toString(ENCODING);

            /*
            SOSIUtil s = new SOSIUtil(false, true);
            ProxyData p = s.validateDGWS(xml, 4);
            System.out.println("ProxyData: " + p);
             */

            System.out.println("Request: " + xml);
            String response = httpWrite("http://localhost:8080/fsk/services/fsk", "urn:ihe:iti:2007:RetrieveDocumentSet", xml, null);
            System.out.println("Response: " + response);

        } catch (Throwable ex) {
            log.error("test callOnDemandService failed", ex);
        }
    }

    private String httpWrite(String endpoint, String soapAction, String xmldata, String proxyHeaderJson) throws Exception {
        HttpClient httpClient = new DefaultHttpClient();

        System.out.println(" Calling " + endpoint + ", SOAPAction=" + soapAction);
        HttpPost request = new HttpPost(endpoint);

        if (soapAction != null) {
            request.addHeader("SOAPAction", "\"" + soapAction + "\"");
        }

        if (proxyHeaderJson != null) {
            request.addHeader("X-Proxy-Data", proxyHeaderJson);
        }

        StringEntity stringEntity = new StringEntity(xmldata, ENCODING);
        stringEntity.setContentType("text/xml");
        request.setEntity(stringEntity);

        HttpResponse response = httpClient.execute(request);
        return EntityUtils.toString(response.getEntity(), ENCODING);
    }

}
