/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations;

import dk.stamkort.dao.exceptions.DDSException;
import dk.stamkort.dao.exceptions.DDSPermanentException;
import dk.stamkort.dao.exceptions.DDSTemporaryException;
import dk.stamkort.integrations.dds.DDSRegisterOnDemandClient;
import dk.stamkort.util.RequestContextVO;
import oasis.names.tc.ebxml_regrep.xsd.rs._3.RegistryResponseType;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.test.context.junit4.SpringRunner;

import javax.xml.ws.Endpoint;

import static dk.stamkort.integrations.dds.XdsDocumentEntryConstants.XDS_STATUS_SUCCESS;
import static junit.framework.TestCase.assertNull;
import static org.junit.Assert.*;


@RunWith(SpringRunner.class)
@SpringBootTest
public class DDSRegisterOnDemandClientTests {

    public static final String ENDPOINT = "http://localhost:8085/";

    @Mock
    private RequestContextVO requestContext;

    @Autowired
    private DDSRegisterOnDemandClient client;

    @Test
    public void testOkResponse() throws DDSException {
        client.setEndpoint(ENDPOINT);
        Resource resource = new ClassPathResource("testfiles/DDSSubmitObjectsResponse.xml");
        TestServer testServer = new TestServer(resource);
        Endpoint endpoint =  Endpoint.publish(ENDPOINT, testServer);

        RegistryResponseType response = client.registerOnDemandDocument("", "");
        endpoint.stop();
        assertNotNull(response);
        assertNull(response.getRegistryErrorList());
        assertEquals(XDS_STATUS_SUCCESS, response.getStatus());
    }

    @Test
    public void testResponseError() throws DDSTemporaryException {
        client.setEndpoint(ENDPOINT);
        Resource resource = new ClassPathResource("testfiles/DDSSubmitObjectsResponseError.xml");
        TestServer testServer = new TestServer(resource);
        Endpoint endpoint =  Endpoint.publish(ENDPOINT, testServer);

        try {
            RegistryResponseType response = client.registerOnDemandDocument("", "");
            fail("Expected DDSPermanentException");
        } catch (DDSPermanentException e) {
            assertEquals("Unexpected DDS response: [Duplicate ID found]", e.getMessage());
        } finally {
            endpoint.stop();
        }
    }

}
