/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations;

import dk.stamkort.dao.exceptions.DDSException;
import dk.stamkort.dao.exceptions.DDSPermanentException;
import dk.stamkort.integrations.dds.DDSRegistryStoredQueryClient;
import dk.stamkort.util.RequestContextVO;
import oasis.names.tc.ebxml_regrep.xsd.query._3.AdhocQueryResponse;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.test.context.junit4.SpringRunner;

import javax.xml.ws.Endpoint;

import static dk.stamkort.integrations.dds.XdsDocumentEntryConstants.XDS_STATUS_SUCCESS;
import static junit.framework.TestCase.assertNull;
import static org.junit.Assert.*;


@RunWith(SpringRunner.class)
@SpringBootTest
public class DDSRegistryStoredQueryClientTests {

    public static final String ENDPOINT = "http://localhost:8085/";

    @Mock
    private RequestContextVO requestContext;

    @Autowired
    private DDSRegistryStoredQueryClient client;

    @Test
    public void testOkDocumentByIdResponse() throws DDSException {
        client.setEndpoint(ENDPOINT);
        Resource resource = new ClassPathResource("testfiles/DDSRegistryStoredQueryResponse.xml");
        TestServer testServer = new TestServer(resource);
        Endpoint endpoint =  Endpoint.publish(ENDPOINT, testServer);

        AdhocQueryResponse response = client.getDocumentById("foo");
        endpoint.stop();
        assertNotNull(response);
        assertNull(response.getRegistryErrorList());
        assertEquals(XDS_STATUS_SUCCESS, response.getStatus());

        assertEquals(1, response.getRegistryObjectList().getIdentifiable().size());
    }

    @Test
    public void testOkRegistryIdByFSKDocumentIdResponse() throws DDSException {
        client.setEndpoint(ENDPOINT);
        Resource resource = new ClassPathResource("testfiles/DDSRegistryStoredQueryResponse.xml");
        TestServer testServer = new TestServer(resource);
        Endpoint endpoint =  Endpoint.publish(ENDPOINT, testServer);

        String response = client.getRegistryIdByFSKDocumentId("foo");
        endpoint.stop();
        assertEquals("urn:uuid:1da1afa6-04ca-4f8d-a8e2-eaf93d10081a", response);
    }

    @Test
    public void testRegistryIdByFSKDocumentIdErrorResponse() throws DDSException {
        client.setEndpoint(ENDPOINT);
        Resource resource = new ClassPathResource("testfiles/DDSRegistryStoredQueryResponseError.xml");
        TestServer testServer = new TestServer(resource);
        Endpoint endpoint =  Endpoint.publish(ENDPOINT, testServer);

        try {
            client.getRegistryIdByFSKDocumentId("foo");
            fail("Expected DDSPermanentException");
        } catch (DDSPermanentException e) {
            assertEquals("Unexpected DDS result size for documentId=foo. Expected 1, was: 2", e.getMessage());
        } finally {
            endpoint.stop();
        }
    }

}
