/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations.dds;

import dk.nsi.documentsharing.consumer.DocumentConsumer;
import dk.nsi.documentsharing.consumer.helper.DocumentSharingHelper;
import dk.nsi.documentsharing.core.metadata.DocumentMetadata;
import dk.nsi.documentsharing.core.metadata.SimpleDocumentMetadata;
import dk.nsi.documentsharing.core.query.DocumentMetadataQueryResult;
import dk.nsi.documentsharing.core.query.FindDocumentsQuery;
import dk.nsi.documentsharing.core.retrieve.DocumentsToRetrieve;
import dk.nsi.documentsharing.core.retrieve.RetrievedDocumentsResult;
import dk.stamkort.util.Properties;
import oasis.names.tc.ebxml_regrep.xsd.rs._3.RegistryResponseType;
import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.Collections;
import java.util.List;
import java.util.UUID;

/**
 * This class contains (ignored) utility tests that can be used for testing DDS/registry functionality. Should not be run
 * as part of usual tests because they require access to DDS.
 */
@RunWith(SpringRunner.class)
@SpringBootTest
public class DDSHoleThroughTest {

    @Autowired
    Properties properties;

    @Autowired
    DDSRegisterOnDemandClient ddsRegister;

    @Autowired
    DDSRegistryStoredQueryClient ddsQuery;

    @Autowired
    DDSRegistryUpdateClient ddsUpdate;

    @Test
    @Ignore // See class JavaDoc
    public void callDDSRegisterOnDemandDocument() throws Exception {
        String fskDocumentId = "urn:sds:fsk-ohe:stamkort:" + UUID.randomUUID().toString();
        RegistryResponseType registryResponseType = ddsRegister.registerOnDemandDocument(fskDocumentId, "1122334455");
        System.out.println(registryResponseType);
    }

    @Test
    @Ignore // See class JavaDoc
    public void callDDSRegistryStoredQuery() throws Exception {
        String id = ddsQuery.getRegistryIdByFSKDocumentId("1e3777cc-3010-49de-8d19-3b71c35d6f9a");
        System.out.println("ddsID=" + id);
    }

    @Test
    @Ignore // See class JavaDoc
    public void callDDSRegistryUpdate() throws Exception {
        ddsUpdate.deprecateDocumentUsingDdsId("1122334455", "urn:uuid:1da1afa6-04ca-4f8d-a8e2-eaf93d10081a");
    }

    @Test
    @Ignore // See class JavaDoc
    public void callDDSRegistryGetDocument() throws Exception {

        // Client configuration
        System.setProperty("testclient.properties", "src/test/resources/testclient.properties");

        // Query DDS metadata
        String testCPR = "1122334455";
        DocumentConsumer docConsumer = DocumentSharingHelper.getConsumer();
        FindDocumentsQuery query = docConsumer.createQuery(DocumentSharingHelper.getNationalProfileValueFactory()
                .createPatientIdUsingCpr(testCPR)).addQueryForOnDemandDocuments();
        DocumentMetadataQueryResult queryResult = docConsumer.doQuery(query);

        // Look up actual document
        String documentId = "urn:sds:fsk-ohe:stamkort:5b7c22b9-5e48-49a8-a035-491b26d91793";
        List<? extends SimpleDocumentMetadata> metadata = getByRepositoryUniqueIdAndDocumentId(queryResult, properties.getDdsRepositoryUniqueId(), documentId);
        DocumentsToRetrieve fetchList = docConsumer.prepareRetrieve(metadata);
        RetrievedDocumentsResult retrievedDocuments = docConsumer.doRetrieve(fetchList);

        outputResult(retrievedDocuments);
    }

    private void outputResult(RetrievedDocumentsResult retrievedDocuments) {
        System.out.println("Errors: " + retrievedDocuments.getDocumentSharingErrorList().getErrors().size());
        System.out.println("Warnings: " + retrievedDocuments.getDocumentSharingErrorList().getWarnings().size());
        System.out.println("Retrieved documents: " + (retrievedDocuments.getDocuments() != null ? retrievedDocuments.getDocuments().size() : 0));
        System.out.println("Unreachable documents: " + (retrievedDocuments.getUnreachableDocuments() != null ? retrievedDocuments.getUnreachableDocuments().size() : 0));
        System.out.println(ReflectionToStringBuilder.toString(retrievedDocuments, ToStringStyle.MULTI_LINE_STYLE));
    }

    public static List<? extends SimpleDocumentMetadata> getByRepositoryUniqueIdAndDocumentId(DocumentMetadataQueryResult queryResult, String repositoryUniqueId, String documentId) {
        for (DocumentMetadata metadata : queryResult.getDocumentMetadataList()) {
            System.out.println("RepositoryUniqueId: " + metadata.getRepositoryUniqueId());
            if (repositoryUniqueId.equals(metadata.getRepositoryUniqueId()) && documentId.equals(metadata.getDocumentUniqueId().getId())) {
                return Collections.singletonList(metadata);
            }
        }
        throw new RuntimeException("Didn't find any documents");
    }
}
