/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.integrations.dds;

import dk.nsi.documentsharing.consumer.DocumentConsumer;
import dk.nsi.documentsharing.consumer.helper.DocumentSharingHelper;
import dk.nsi.documentsharing.core.metadata.impl.SimpleDocumentMetadataImpl;
import dk.nsi.documentsharing.core.metadata.model.ValueWithAssigningAuthorityImpl;
import dk.nsi.documentsharing.core.retrieve.DocumentsToRetrieve;
import dk.nsi.documentsharing.core.retrieve.RetrievedDocumentsResult;
import dk.nsi.documentsharing.id.IdImpl;
import dk.stamkort.OID;
import dk.stamkort.util.Properties;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.Collections;

/**
 * Utility for performing verification
 */
@RunWith(SpringRunner.class)
@SpringBootTest
public class DDSIntegrationTest {

    // Use values from table RegistryIndex
    String personIdentifier = "1111111118";
    String fskDocumentId = "urn:sds:fsk:stamkort:911c0b38-546a-46f4-b4cf-8cde300106e9";

    @Autowired
    Properties properties;

    @Test
    @Ignore // Not to be run as part of unit tests because it requires access to DDS
    public void callDDS() throws Exception {
        System.setProperty("testclient.properties", "src/test/resources/testclient.properties");

        DocumentConsumer docConsumer = DocumentSharingHelper.getConsumer();
        SimpleDocumentMetadataImpl md = new SimpleDocumentMetadataImpl();
        md.setDocumentUniqueId(new IdImpl(fskDocumentId));
        md.setPatientId(new ValueWithAssigningAuthorityImpl(personIdentifier, OID.CPR));
        md.setHomeCommunityId(properties.getDdsHomeCommunityId());
        md.setRepositoryUniqueId(properties.getDdsRepositoryUniqueId());

        DocumentsToRetrieve fetchList = docConsumer.prepareRetrieve(Collections.singletonList(md));
        RetrievedDocumentsResult retrievedDocuments = docConsumer.doRetrieve(fetchList);

        outputResult(retrievedDocuments);
    }

    private void outputResult(RetrievedDocumentsResult retrievedDocuments) {
        System.out.println("Errors: " + retrievedDocuments.getDocumentSharingErrorList().getErrors().size());
        System.out.println("Warnings: " + retrievedDocuments.getDocumentSharingErrorList().getWarnings().size());
        System.out.println("Retrieved documents: " + (retrievedDocuments.getDocuments() != null ? retrievedDocuments.getDocuments().size() : 0));
        System.out.println("Unreachable documents: " + (retrievedDocuments.getUnreachableDocuments() != null ? retrievedDocuments.getUnreachableDocuments().size() : 0));
        System.out.println("Returned document: " + new String(retrievedDocuments.getDocuments().get(0).getDocument()));
    }

}
