/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.hl7cda.assembler;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.InputStream;
import java.time.format.DateTimeFormatter;
import java.time.format.ResolverStyle;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.hl7.v3.POCDMT000040ClinicalDocument;
import org.junit.Test;

import dk.stamkort.integrations.RegisterData;
import dk.sundhedsdatastyrelsen.fsk._2019._08._01.GetPersonalDataCardResponse;

public class ClinicalDocumentAssemblerTests {
	
	private static final String TS_FORMAT = "yyyyMMddHHmmssZ"; 

    @Test
    public void testPersonalDataCardAssembling() throws JAXBException {
        InputStream stream = getClass().getResourceAsStream("/testfiles/GetPersonalDataCard_response.xml");
        JAXBContext jaxbContext = JAXBContext.newInstance(GetPersonalDataCardResponse.class);
        Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
        GetPersonalDataCardResponse personalDataCardResponse = (GetPersonalDataCardResponse) unmarshaller.unmarshal(stream);

        String cprnr = personalDataCardResponse.getClinicalDocument().getRecordTargets().get(0).getPatientRole().getIds().get(0).getExtension(); // phew...
        RegisterData registerData = new RegisterData(cprnr);
        registerData.setPersonalDataCard(personalDataCardResponse);

        PDC_1_0ClinicalDocumentAssembler assembler = new PDC_1_0ClinicalDocumentAssembler();
        POCDMT000040ClinicalDocument assembledDocument = assembler.assembleClinicalDocument(registerData, "");

        assertNotNull(assembledDocument);
        
        DateValidator validator = new DateValidator(DateTimeFormatter.ofPattern(TS_FORMAT).withResolverStyle(ResolverStyle.STRICT));
      	assertTrue(validator.isValid(assembledDocument.getEffectiveTime().getValue()));

    }

    @Test
    public void testIsEmail() {
        PDC_1_0ClinicalDocumentAssembler assembler = new PDC_1_0ClinicalDocumentAssembler();
        assertFalse(assembler.isEmail(null));
        assertFalse(assembler.isEmail(""));
        assertFalse(assembler.isEmail("foo"));
        assertFalse(assembler.isEmail("foo.com"));
        assertFalse(assembler.isEmail("me@ mydomain.com"));
        assertFalse(assembler.isEmail("me @mydomain.com"));
        assertFalse(assembler.isEmail(" me@mydomain.com"));
        assertFalse(assembler.isEmail("kontakt os på me@mydomain.com"));
        assertTrue(assembler.isEmail("me@mydomain.com"));
        assertTrue(assembler.isEmail("me.2@mydomain.com"));
    }

}
