/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort.webservice.ondemand;

import com.sun.xml.messaging.saaj.soap.ver1_2.Message1_2Impl;
import dk.stamkort.util.FSKException;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.anyObject;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@RunWith(SpringRunner.class)
@SpringBootTest
@Transactional
public class DocumentProviderWSHandlerTest {

    DocumentProviderWSHandler wsHandler = new DocumentProviderWSHandler();

    @Test
    public void testMissingSecurityHeaders() {
        SOAPMessageContext context = mock(SOAPMessageContext.class);
        when(context.get(anyObject())).thenReturn(new Boolean(false));
        when(context.getMessage()).thenReturn(new Message1_2Impl());
        context.setMessage(new Message1_2Impl());
        context.put(MessageContext.MESSAGE_OUTBOUND_PROPERTY, new Boolean(false));
        try {
            wsHandler.handleMessage(context);
            fail("Expected FSKException");
        } catch (FSKException e) {
            assertEquals(500, e.getFaultCode().getCode());
            assertEquals("DGWS/IDWS headers er ikke gyldige.", e.getFaultCode().getMessage());
        }
    }

}
