/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.stamkort;

import dgws.SosiUtil;
import dk.sosi.seal.model.AuthenticationLevel;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.hl7.v3.POCDMT000040ClinicalDocument;
import org.junit.Test;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.stream.StreamSource;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.StringReader;

import static org.junit.Assert.*;

public class OnDemandServiceITCase {
    private static final String ENCODING = "UTF-8";

    private String host = "localhost";
    private String port = "8080";
    private String docid = null;

    public OnDemandServiceITCase() {
        String host = System.getProperty("FSK_TEST_HOST");
        String port = System.getProperty("FSK_TEST_PORT");
        String docid = System.getProperty("FSK_TEST_DOCID");
        if (host != null || port != null) {
            if (host != null) {
                this.host = host;
            }
            if (port != null) {
                this.port = port;
            }
            if (docid != null) {
                this.docid = docid;
            }
        }
    }

    @Test
    public void callOnDemandService_PDC_1_0ClinicalDocument_() throws Exception {
        InputStream stream = getClass().getResourceAsStream("/testrequest.xml");
        SOAPMessage soapMessage = MessageFactory.newInstance().createMessage(null, stream);

        System.out.println("Initializing SosiUtil");
        try {
            SosiUtil sosiUtil = new SosiUtil();
            System.out.println("SosiUtil initialized");
            sosiUtil.addSoapHeader(soapMessage, AuthenticationLevel.MOCES_TRUSTED_USER);
        } catch (Throwable ex) {
            System.err.println("SosiUtil initialization failed");
            ex.printStackTrace();
        }

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        soapMessage.writeTo(baos);
        String xml = baos.toString(ENCODING);

        if (docid != null) {
            xml = xml.replace("urn:sds:fsk:stamkort:00000000-0000-0000-0000-000000000000", docid);
        }

        System.out.println("Request: " + xml);
        String response = httpWrite("urn:ihe:iti:2007:RetrieveDocumentSet", xml);
        System.out.println("Response: " + response);

        assertNotNull(response);
        assertTrue(response.contains("ClinicalDocument"));
    }

    @Test
    public void callOnDemandService_PDC_2_0ClinicalDocument_correctIds() throws Exception {
        InputStream stream = getClass().getResourceAsStream("/testrequest2.xml");
        SOAPMessage soapMessage = MessageFactory.newInstance().createMessage(null, stream);

        System.out.println("Initializing SosiUtil");
        try {
            SosiUtil sosiUtil = new SosiUtil();
            System.out.println("SosiUtil initialized");
            sosiUtil.addSoapHeader(soapMessage, AuthenticationLevel.MOCES_TRUSTED_USER);
        } catch (Throwable ex) {
            System.err.println("SosiUtil initialization failed");
            ex.printStackTrace();
        }

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        soapMessage.writeTo(baos);
        String xml = baos.toString(ENCODING);

        if (docid != null) {
            xml = xml.replace("urn:sds:fsk:stamkort:00000000-0000-0000-0000-000000000001", docid);
        }

        System.out.println("Request: " + xml);
        String responseA = httpWrite("urn:ihe:iti:2007:RetrieveDocumentSet", xml);
        System.out.println("Response: " + responseA);

        String xmlResponseA = responseA.substring(responseA.indexOf("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ClinicalDocument moodCode=\"EVN\""), responseA.indexOf("</ClinicalDocument>")+"</ClinicalDocument>".length());
        assertNotNull(responseA);
        assertTrue(responseA.contains("ClinicalDocument"));

        JAXBContext contextA = JAXBContext.newInstance(POCDMT000040ClinicalDocument.class);
        JAXBElement<POCDMT000040ClinicalDocument> rA = contextA.createUnmarshaller().unmarshal(new StreamSource(new StringReader(xmlResponseA)), POCDMT000040ClinicalDocument.class);

        String languageIdA = rA.getValue().getComponent().getStructuredBody().getComponents().get(0).getSection().getEntries().get(8).getObservation().getIds().get(0).getExtension();
        String tempAddrIdA = rA.getValue().getComponent().getStructuredBody().getComponents().get(0).getSection().getEntries().get(9).getObservation().getIds().get(0).getExtension();
        String dentistIdA = rA.getValue().getComponent().getStructuredBody().getComponents().get(0).getSection().getEntries().get(10).getObservation().getIds().get(0).getExtension();

        // Run again
        String responseB = httpWrite("urn:ihe:iti:2007:RetrieveDocumentSet", xml);

        String xmlResponseB = responseB.substring(responseB.indexOf("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ClinicalDocument moodCode=\"EVN\""), responseB.indexOf("</ClinicalDocument>")+"</ClinicalDocument>".length());
        assertNotNull(responseB);
        assertTrue(responseB.contains("ClinicalDocument"));

        JAXBContext contextB = JAXBContext.newInstance(POCDMT000040ClinicalDocument.class);
        JAXBElement<POCDMT000040ClinicalDocument> rB = contextB.createUnmarshaller().unmarshal(new StreamSource(new StringReader(xmlResponseB)), POCDMT000040ClinicalDocument.class);

        String languageIdB = rB.getValue().getComponent().getStructuredBody().getComponents().get(0).getSection().getEntries().get(8).getObservation().getIds().get(0).getExtension();
        String tempAddrIdB = rB.getValue().getComponent().getStructuredBody().getComponents().get(0).getSection().getEntries().get(9).getObservation().getIds().get(0).getExtension();
        String dentistIdB = rB.getValue().getComponent().getStructuredBody().getComponents().get(0).getSection().getEntries().get(10).getObservation().getIds().get(0).getExtension();

        // Assert the same Ids are provided in both calls
        assertTrue(tempAddrIdA.equals(tempAddrIdB));
        assertTrue(dentistIdA.equals(dentistIdB));

        //SKR returnerer ikke det samme Language id hver gang den kaldes, men i stedet et af tre forskellige id'er for denne test person.
        assertTrue(languageIdA.equals("fc9dd3b9-e308-4b43-a7f8-a692abca6623") || languageIdA.equals("fc9dd3b9-e308-4b43-a7f8-a692abca6622") || languageIdA.equals("5477653c-4c46-4f50-a566-591d677b1111"));
        assertTrue(languageIdB.equals("fc9dd3b9-e308-4b43-a7f8-a692abca6623") || languageIdB.equals("fc9dd3b9-e308-4b43-a7f8-a692abca6622") || languageIdB.equals("5477653c-4c46-4f50-a566-591d677b1111"));
    }

    private String httpWrite(String soapAction, String xmldata) throws Exception {
        HttpClient httpClient = new DefaultHttpClient();

        String endpoint = "http://" + host + ":" + port + "/fsk/services/fsk";
        System.out.println(" Calling " + endpoint + ", SOAPAction=" + soapAction);
        HttpPost request = new HttpPost(endpoint);
        request.addHeader("SOAPAction", "\"" + soapAction + "\"");

        StringEntity stringEntity = new StringEntity(xmldata, ENCODING);
        stringEntity.setContentType("text/xml");
        request.setEntity(stringEntity);

        HttpResponse response = httpClient.execute(request);
        return EntityUtils.toString(response.getEntity(), ENCODING);
    }
}
